<div class="wrap">
	<h1>Chained Logic for Quizzes</h1>
	
	<p>You can enable chained logic for any of your quizzes. This lets you direct the user to different questions depending on the answers they give. Once you enable chained logic for a given quiz, the following things will happen:</p>
	
	<ol>
		<li>The quiz pagination will be automatically set up to "One question per page"</li>
		<li>Back buttons, "save" button, numbered paginators, "show answer", and "Question X from Y" text will not be shown.</li>
		<li>Grouping and randomizing questions settings will be disregarded - the questions will start appearing by the order you have given in the "Manage questions" page.</li>
		<li>This will work for "live quizzes" (no "practice mode")</li>
		<li>Only single-answer, multiple-answer and "open-end" questions can be chained. The other question types will be excluded.</li>
		<li>Chained quizzes partially-submitted state cannot be saved so "automatically store user progress as they go from page to page" will not work.</li>
	</ol>
	
	<p>It is <b>your responsibility</b> to avoid endless loops.</p>
	
	<form method="post">
		<p>Set text for the next question button: <input type="text" name="button_text" value="<?php echo $button_text ? $button_text : 'Go Ahead'?>"><br>
		  <input type="checkbox" name="required_from_watupro" value="1" <?php if(get_option('wchained_required_from_watupro') == 1) echo 'checked'?> onclick="this.checked ? jQuery('#prevAnswerText').show() : jQuery('#prevAnswerText').hide();"> <?php _e('Require answering the questions only when they are marked as required in WatuPRO (works with WatuPRO version 6.2.0.3 or newer). Otherwise all questions are required by default.', 'wchained');?></p>
		  
		 <p> <input type="checkbox" name="enable_log" value="1" <?php if($enable_log == 1) echo 'checked'?>> <?php _e('Enable debug log', 'wchained');?>
			<?php if(!empty($enable_log)):?>
				<a href="#" onclick="jQuery('#wchainedDebugLog').toggle();return false;"><?php _e('View debug log (up to latest 1000 entries)', 'wchained');?></a>
				<table class="widefat" style="display: none;" id="wchainedDebugLog">
					<tr><th><?php _e('Date/Time', 'wchained');?></th><th><?php _e('Question ID:', 'wchained');?></th><th><?php _e('Log contents','wchained');?></th></tr>
					<?php foreach($logs as $log):
						if(empty($class)) $class = 'alternate';
						else $class = '';?>
						<tr class="<?php echo $class;?>">
							<td><?php echo date_i18n($date_format, $log->datetime);?></td>
							<td><?php echo $log->question_id;?></td>
							<td><?php echo $log->log;?></td>
						</tr>
					<?php endforeach;?>					
				</table>
			<?php endif;?>		 
		 </p> 
		<input type="submit" class="button button-primary" value="Save" name="save_button_text"></p>
		<?php wp_nonce_field('wchained_options');?>
	</form>
	
	<form method="post">
		<p><?php _e('Select quiz:', 'wtpuc')?> <select name="quiz_id" onchange="window.location='admin.php?page=wchained_options&quiz_id='+this.value;">
			<option value="0">Select Quiz</option>
			<?php foreach($quizzes as $quiz):
				$selected = ($quiz->ID == @$_GET['quiz_id']) ? 'selected' : '';
				if(!empty($quiz->cat)) $quiz->name = $quiz->cat.': '.$quiz->name;?>
				<option value="<?php echo $quiz->ID?>" <?php echo $selected?>><?php echo stripslashes($quiz->name)?></option>
			<?php endforeach;?>	
		</select></p>
			
		<?php if(!empty($_GET['quiz_id'])):?>
			<p>Use this shortcode to publish the chained version of the quiz: <input type="text" value="[watupro_chained quiz_id=<?php echo $_GET['quiz_id']?>]" readonly="readonly" onclick="this.select();" size="30"></p>
			
			<?php foreach($questions as $question):?>
				<h2>Question ID: <?php echo $question->ID?></h2>
				<?php echo wpautop(empty($question->title) ? stripslashes($question->question) : stripslashes($question->title));?>
				<p>Question type: <b><?php echo $question->answer_type?></b></p>
					<?php if(count($question->choices)):?>			
					<h3>Question answers:</h3>
					<table class="widefat">
						<tr><th>Answer</th><th>Correct?</th><th>Points</th><th>When selected, go to:</th></tr>
						<?php foreach($question->choices as $cnt=>$choice):
							$class = ('alternate' == @$class) ? '' : 'alternate';?>
							<input type="hidden" name="choice_ids[]" value="<?php echo $choice->ID?>">
							<tr class="<?php echo $class?>">
							<td><?php echo wpautop(stripslashes($choice->answer));?></td>
							<td><?php echo $choice->correct ? 'Yes' : 'No'?></td>
							<td><?php echo $choice->point?></td>
							<td><select name="<?php echo 'goto'.$choice->ID?>">
								<option value="next" <?php if(!empty($choice->chain_goto) and $choice->chain_goto == 'next') echo 'selected'?>>Next question</option>
								<option value="finalize" <?php if(!empty($choice->chain_goto) and $choice->chain_goto == 'finalize') echo 'selected'?>>Finalize quiz</option>
									<option disabled><?php _e('- Select question -', 'chained')?></option>
									<?php foreach($questions as $other_question):
										//if($other_question->ID == $question->ID) continue;?>
										<option value="<?php echo $other_question->ID?>" <?php if(!empty($choice->chain_goto) and $choice->chain_goto == $other_question->ID) echo 'selected'?>><?php echo wchained_summary($other_question);?></option>
									<?php endforeach;?>								
								</select> </td></tr>
						<?php endforeach; // end foreach choice ?>
					</table>
					<?php endif;?>
					<p><input type="checkbox" name="use_logic<?php echo $question->ID?>" value="1" <?php if(!empty($question->chain_use_logic)) echo 'checked'?> onclick="this.checked ? jQuery('#wchainedUseLogic<?php echo $question->ID?>').show() : jQuery('#wchainedUseLogic<?php echo $question->ID?>').hide();"> Use dynamic logic based on percent correct answers after answering the question.</p>
					<div id="wchainedUseLogic<?php echo $question->ID?>" style='display:<?php echo empty($question->chain_use_logic)? 'none' : 'block'?>'>
						<table><tr><td><input type="radio" name="logic_mode<?php echo $question->ID?>" value="percent" <?php if(empty($question->chain_logic_mode) or $question->chain_logic_mode == 'percent') echo 'checked';?>> If the % correct answers from all questions up to this point is <select name="logic_trigger<?php echo $question->ID?>">
							<option value="above">above</option>
							<option value="below" <?php if(!empty($question->chain_logic_trigger) and $question->chain_logic_trigger == 'below') echo 'selected'?>>below</option>
						</select> 
						<input type="text" name="logic_percent<?php echo $question->ID?>" size="5" value="<?php echo @$question->chain_percent?>">% 
							<input type="checkbox" name="logic_samecat<?php echo $question->ID?>" value="1" <?php echo !empty($question->chain_samecat) ? 'checked' : '';?>> <i><?php _e('in the same question category');?></i>
							<br>
							<input type="radio" name="logic_mode<?php echo $question->ID?>" value="question" <?php if(!empty($question->chain_logic_mode) and $question->chain_logic_mode == 'question') echo 'checked';?>> If the answer to this specific question is <select name="logic_is_correct<?php echo $question->ID?>">
							<option value="correct">correct</option>
							<option value="wrong" <?php if(!empty($question->chain_logic_trigger) and $question->chain_logic_trigger == 'wrong') echo 'selected'?>>wrong</option>
						</select> 
							</td>
							<td width="100" align="center"><h2>&gt;&gt;&gt;</h2></td>
							<td>
							go to:		
							<select name="<?php echo 'logic_goto'.$question->ID?>">
									<option value="next" <?php if(!empty($question->chain_goto) and $question->chain_goto == 'next') echo 'selected'?>>Next question</option>
									<option value="finalize" <?php if(!empty($question->chain_goto) and $question->chain_goto == 'finalize') echo 'selected'?>>Finalize quiz</option>
										<option disabled><?php _e('- Select question -', 'chained')?></option>
										<?php foreach($questions as $other_question):
											//if($other_question->ID == $question->ID) continue;?>
											<option value="<?php echo $other_question->ID?>" <?php if(!empty($question->chain_goto) and $question->chain_goto == $other_question->ID) echo 'selected'?>><?php echo wchained_summary($other_question);?></option>
										<?php endforeach;?>								
							</select>
							(Otherwise follow the logic set above this rule)	
							</td></tr></table>
						</div>	
					
				<hr>
			<?php endforeach; // end foreach question ?>
			
			 <p><input type="checkbox" name="display_previous_answer" value="1" <?php if(!empty($prev_answer_text)) echo 'checked'?> onclick="this.checked ? jQuery('#prevAnswerText').show() : jQuery('#prevAnswerText').hide();"> <?php _e('Display previous answer before the question', 'wchained');?></p>			
				 <div id="prevAnswerText" style='display:<?php echo empty($prev_answer_text) ? 'none':'block';?>'>
				     <textarea rows="2" cols="60" name="prev_answer_text"><?php 
				     echo $prev_answer_text ? stripslashes($prev_answer_text) : "You answered: {{{prev-answer}}}";
				     ?></textarea><br>
				     Please keep the variable {{{prev-answer}}} inside the box above. It will be replaced with the answer to the previous question.
				 </div>
				<hr> 
			<p><input type="submit" value="Save Chained Quiz Logic" class="button button-primary"></p>
			<input type="hidden" name="ok" value="1">
		<?php endif;?>
		<?php wp_nonce_field('wchained_logic');?>
	</form>	
	
	<?php if(!empty($chained_quizzes) and sizeof($chained_quizzes)):?>
		<h2>Transfer Chained Quizzes</h2>
		<?php if(!empty($success)) echo "<p><b>$success</b></p>";?>
		
		<p>You have <?php echo sizeof($chained_quizzes)?> chained quizzes made with the free Chained Quiz plugin. Do you want to have these transferred to WatuPRO? You can do this here:</p>
		<form method="post">
		<p><input type="checkbox" name="change_shortcodes" value="1"> When transferring published quizzes, also change their shortcodes to use Chained logic for WatuPRO.</p>
		
		<p><input type="submit" value="Yes, transfer these quizzes!" name="transfer_chained" class="button button-primary"></p>
		</form>
	<?php endif;?>

</div>