<?php
/*
Plugin Name: Chained Logic for WatuPRO
Plugin URI: 
Description: Adds Chained Quiz logic for WatuPRO
Author: Kiboko Labs
Version: 4.3
Author URI: http://calendarscripts.info/
License: GPLv2 or later
*/

define( 'WCHAINED_PATH', dirname( __FILE__ ) );
define( 'WCHAINED_RELATIVE_PATH', dirname( plugin_basename( __FILE__ )));
define( 'WCHAINED_URL', plugin_dir_url( __FILE__ ));

require(WCHAINED_PATH.'/controllers/shortcodes.php');
require(WCHAINED_PATH.'/controllers/ajax.php');
require(WCHAINED_PATH.'/controllers/quizzes.php');

register_activation_hook( __FILE__, 'wchained_activate' );
add_action('init', 'wchained_init');

function wchained_activate($update = false) {
	// let's not use this for now and see how things work only with using sessions
	
	global $user_ID, $wpdb;
	if(!$update) wchained_init();
	
	// this stores the chained logic of the answers in WatuPRO
	 if($wpdb->get_var("SHOW TABLES LIKE '".WCHAINED_CHAINS."'") != WCHAINED_CHAINS) {  
         $sql = "CREATE TABLE `".WCHAINED_CHAINS."` (
						`id` int(11) unsigned NOT NULL auto_increment,
						`exam_id` int(11) unsigned NOT NULL DEFAULT '0',						
						`choice_id` int(11) unsigned NOT NULL DEFAULT '0',
						`goto` VARCHAR(100) NOT NULL DEFAULT 'next',
						PRIMARY KEY  (ID)						
					) CHARACTER SET utf8;";
         $wpdb->query($sql);    
     }
     
     // add logic threshold fields
    watupro_add_db_fields(array(
    		array("name"=>"question_id", "type"=>"INT UNSIGNED NOT NULL DEFAULT 0"), /* when the chain is about the question */
   		array("name"=>"use_logic", "type"=>"TINYINT UNSIGNED NOT NULL DEFAULT 0"), /* use logic threshold? */
   		array("name"=>"logic_percent", "type"=>"TINYINT UNSIGNED NOT NULL DEFAULT 0"), /* required % correct answers */
   		array("name"=>"logic_trigger", "type"=>"VARCHAR(100) NOT NULL DEFAULT 'above'"), /* above or below the % correct */
   		array("name"=>"logic_goto", "type"=>"VARCHAR(100) NOT NULL DEFAULT 'next'"), /* same as the question goto */
   		array("name"=>"logic_samecat", "type"=>"TINYINT UNSIGNED NOT NULL DEFAULT 0"), /* whether the logic is only for the same question category */
   		array("name"=>"logic_mode", "type"=>"VARCHAR(100) NOT NULL DEFAULT 'percent'"), /* defaults to % correct answers but now we have "when this answer is corect/wrong" */
		), WCHAINED_CHAINS); 
		
	watupro_add_db_fields(array(
	    array("name"=>"chained_evaluations", "type"=>"TEXT"), /* serialized $_SESSION['wchained_evaluations_quizID] so we can properly reestore it from in_progress */ 
	), $wpdb->prefix.'watupro_taken_exams');	
		
	// debug logs	
	if($wpdb->get_var("SHOW TABLES LIKE '".WCHAINED_LOGS."'") != WCHAINED_LOGS) {  
         $sql = "CREATE TABLE `".WCHAINED_LOGS."` (
						`id` int(11) unsigned NOT NULL auto_increment,
						`datetime` DATETIME,						
						`question_id` int(11) unsigned NOT NULL DEFAULT '0',
						`log` TEXT,
						PRIMARY KEY  (ID)						
					) CHARACTER SET utf8;";
         $wpdb->query($sql);    
     }	
		
	update_option('wchained_version', '0.28');	
}

function wchained_init() {
	global $wpdb;
	// start session only on front-end and WatuPRO admin pages
	if (!session_id() and ( (!empty($_GET['page']) and (strstr($_GET['page'], 'watupro'))) or !is_admin()
		or (wp_doing_ajax() and !empty($_POST['wchained_action'])) )) {
			@session_start();
	}
	
   define('WCHAINED_CHAINS', $wpdb->prefix.'wchained_chains');
   define('WCHAINED_LOGS', $wpdb->prefix.'wchained_logs');
	
	wp_enqueue_script('jquery');
	
	add_action('watupro_admin_menu', 'wchained_menu');
	add_shortcode('watupro_chained', 'wchained_shortcode');	
	
	// other actions
	add_action('wp_ajax_wchained_ajax', 'wchained_logic_ajax');
	add_action('wp_ajax_nopriv_wchained_ajax', 'wchained_logic_ajax');
	
	$version = get_option('wchained_version');
	if(empty($version) or version_compare($version, '0.28') == -1) wchained_activate(true);
	
	// delete logs older than 30 days
	$last_cleanup = get_option('wchained_last_cleanup');
	if(empty($last_cleanup) or $last_cleanup < time() - 24*3600) {
		 update_option('wchained_last_cleanup', time());
		 
		 $wpdb->query("DELETE FROM ".WCHAINED_LOGS." WHERE datetime < NOW() - INTERVAL 30 DAY");
	}
}

function wchained_options() {
	global $wpdb;
	
	if(!empty($_POST['save_button_text']) and check_admin_referer('wchained_options')) {
		update_option('wchained_button_text', sanitize_text_field($_POST['button_text']));
		$required_from_watupro = empty($_POST['required_from_watupro']) ? 0 : 1;
		update_option('wchained_required_from_watupro', $required_from_watupro);
		
		$enable_log = empty($_POST['enable_log']) ? 0 : 1;
		update_option('wchained_enable_log', $enable_log);
	}
	
	$enable_log = get_option('wchained_enable_log');	
	
	// select all quizzes from WatuPro
	$quizzes = $wpdb->get_results("SELECT tQ.ID, tQ.name, tC.name as cat 
		FROM ".WATUPRO_EXAMS." tQ LEFT JOIN ".WATUPRO_CATS." tC ON tC.ID = tQ.cat_id
		ORDER BY tC.name, tQ.name");	
	
	$prev_answer_text = '';
	
	if(!empty($_GET['quiz_id'])) {
		$_GET['quiz_id'] = intval($_GET['quiz_id']);
		$quiz = $wpdb->get_row($wpdb->prepare("SELECT * FROM ".WATUPRO_EXAMS." WHERE ID=%d", $_GET['quiz_id']));		
		$q_exam_id = $quiz->reuse_questions_from ? $quiz->reuse_questions_from : $quiz->ID; 		
		
		if(!empty($_POST['ok']) and check_admin_referer('wchained_logic')) {
			// save chained logic
			$choice_ids = is_array($_POST['choice_ids']) ? $_POST['choice_ids'] : array();
			
			//print_r($choice_ids);
			foreach($choice_ids as $choice_id) {
				// insert or udpate the chain
				$exists = $wpdb->get_var($wpdb->prepare("SELECT id FROM ".WCHAINED_CHAINS."
					WHERE choice_id=%d AND exam_id=%d", $choice_id, $_GET['quiz_id']));
				if($exists) {
					//echo "exists";
					$wpdb->query($wpdb->prepare("UPDATE ".WCHAINED_CHAINS." SET
						exam_id=%d, choice_id=%d, goto=%s
						WHERE id=%d", $_GET['quiz_id'], $choice_id, $_POST['goto'.$choice_id], $exists));
				}	
				else {
					//echo "new chain";
					$wpdb->query($wpdb->prepare("INSERT INTO ".WCHAINED_CHAINS." SET
						exam_id=%d, choice_id=%d, goto=%s", $_GET['quiz_id'], $choice_id, $_POST['goto'.$choice_id]));
				}
			} // end foreach choice			
			
			// set pagination to one question per page
			$wpdb->query($wpdb->prepare("UPDATE ".WATUPRO_EXAMS." SET single_page=0 WHERE ID=%d", $_GET['quiz_id']));
			
			// save text before each question
			if(!empty($_POST['display_previous_answer'])) {
			   update_option('wchained-logic-prev-text-'.$_GET['quiz_id'], $_POST['prev_answer_text']);
			}
			else delete_option('wchained-logic-prev-text-'.$_GET['quiz_id']);
			
			// save question based % correct logic
			$questions = $wpdb->get_results($wpdb->prepare("SELECT ID, question, answer_type, title
			FROM ".WATUPRO_QUESTIONS." WHERE exam_id=%d AND answer_type IN ('radio', 'checkbox', 'textarea', 'gaps', 'sort', 'slider') 
			AND is_inactive=0	ORDER BY sort_order, ID", $q_exam_id));
			
			foreach($questions as $question) {
				$chain_exists = $wpdb->get_var($wpdb->prepare("SELECT id FROM ".WCHAINED_CHAINS." 
					WHERE question_id=%d AND exam_id=%d", $question->ID, $_GET['quiz_id']));
					
				$use_logic = empty($_POST['use_logic'.$question->ID]) ? 0 : 1;
				$logic_percent = intval($_POST['logic_percent'.$question->ID]);
				$logic_mode = sanitize_text_field($_POST['logic_mode'.$question->ID]);				 
				$logic_trigger = sanitize_text_field($_POST['logic_trigger'.$question->ID]);
				if($logic_mode == 'question') $logic_trigger = sanitize_text_field($_POST['logic_is_correct'.$question->ID]);
				$logic_goto = sanitize_text_field($_POST['logic_goto'.$question->ID]);
				$logic_samecat = empty($_POST['logic_samecat'.$question->ID]) ? 0 : 1;				
				// echo "Question ID {$question->ID} USe logic: $use_logic<br>";
				
				if($use_logic) {
					if($chain_exists) { 
						$wpdb->query($wpdb->prepare("UPDATE " . WCHAINED_CHAINS ." SET
							use_logic=%d, logic_trigger=%s, logic_percent=%d, logic_goto=%s, logic_samecat=%d, logic_mode=%s WHERE id=%d",
							$use_logic, $logic_trigger, $logic_percent, $logic_goto, $logic_samecat, $logic_mode, $chain_exists));
					}
					else {
						$wpdb->query($wpdb->prepare("INSERT INTO " . WCHAINED_CHAINS ." SET
							use_logic=%d, logic_trigger=%s, logic_percent=%d, logic_goto=%s, logic_samecat=%d, logic_mode=%s, exam_id=%d, question_id=%d",
							$use_logic, $logic_trigger, $logic_percent, $logic_goto, $logic_samecat, $logic_mode, intval($_GET['quiz_id']), $question->ID));						
					}	
				}	// end if use logic
				else {
					$wpdb->query($wpdb->prepare("DELETE FROM ".WCHAINED_CHAINS." WHERE question_id=%d AND exam_id=%d", 
						$question->ID, $_GET['quiz_id']));
				} // end if/else use logic
			} // end foreach question
			
		} // end saving
		
		
		// select questions
		$questions = $wpdb->get_results($wpdb->prepare("SELECT tQ.ID, tQ.question, tQ.answer_type, tQ.title as title,
			tC.use_logic as chain_use_logic, tC.logic_percent as chain_percent,
			tC.logic_trigger as chain_logic_trigger, tC.logic_goto as chain_goto, tC.logic_samecat as chain_samecat, tC.logic_mode as chain_logic_mode
			FROM ".WATUPRO_QUESTIONS." tQ LEFT JOIN ".WCHAINED_CHAINS." tC ON tC.question_id = tQ.ID 
			WHERE tQ.exam_id=%d AND tQ.answer_type IN ('radio', 'checkbox', 'textarea', 'gaps', 'sort', 'slider') 
			AND tQ.is_inactive=0
			GROUP BY tQ.ID
			ORDER BY tQ.sort_order, tQ.ID", $q_exam_id));
			
		//print_r($questions);	
			
		$qids = array(0);
		foreach($questions as $question) $qids[] = $question->ID;
		
		// select choices
		$choices = $wpdb->get_results("SELECT * FROM ".WATUPRO_ANSWERS." 
			WHERE question_id IN (".implode(',', $qids).") ORDER BY sort_order, ID");
			
		// select choice chains
		$chains = $wpdb->get_results($wpdb->prepare("SELECT * FROM ".WCHAINED_CHAINS."
			WHERE question_id=0 AND exam_id=%d", $_GET['quiz_id']));	
			
		// match chains to choices	
		foreach($choices as $cnt=>$choice) {
			foreach($chains as $chain) {
				if($chain->choice_id == $choice->ID) {
					$choices[$cnt]->chain_goto = $chain->goto;
				}
			}
		} // end matching chains to choices
		
		// match choices to questions
		foreach($questions as $cnt=>$question) {
			$question_choices = array();
			foreach($choices as $choice) {
				if($choice->question_id == $question->ID) $question_choices[] = $choice;
			}
			
			$questions[$cnt]->choices = $question_choices;
		} // end foreach quiestion
		
		$prev_answer_text = get_option('wchained-logic-prev-text-'.$_GET['quiz_id']);
	} // end if quiz is selected
	
	// quizzes in the free chained quiz plugin?
	if($wpdb->get_var("SHOW TABLES LIKE '".$wpdb->prefix. "chained_quizzes"."'") == $wpdb->prefix. "chained_quizzes") {	
		$chained_quizzes = $wpdb->get_results("SELECT * FROM ".$wpdb->prefix. "chained_quizzes ORDER BY id");
			
		if(!empty($_POST['transfer_chained'])) {
			foreach($chained_quizzes as $chained_quiz) {
				// replace the variables in output
				$chained_quiz->output = str_replace('{{result-title}}', '%%GTITLE%%', $chained_quiz->output);		
				$chained_quiz->output = str_replace('{{result-text}}', '%%GDESC%%', $chained_quiz->output);
				$chained_quiz->output = str_replace('{{points}}', '%%POINTS%%', $chained_quiz->output);
				$chained_quiz->output = str_replace('{{questions}}', '%%TOTAL%%', $chained_quiz->output);
				
				$wpdb->query($wpdb->prepare("INSERT INTO ".WATUPRO_EXAMS." SET
					name=%s, final_screen=%s", stripslashes($chained_quiz->title), stripslashes($chained_quiz->output)));
				$quiz_id = $wpdb->insert_id;
				
				// copy questions and choices
				$questions = $wpdb->get_results($wpdb->prepare("SELECT * FROM {$wpdb->prefix}chained_questions
					WHERE quiz_id=%d ORDER BY rank, id", $chained_quiz->id));
				$choices = $wpdb->get_results($wpdb->prepare("SELECT * FROM {$wpdb->prefix}chained_choices 
					WHERE quiz_id=%d ORDER BY id", $chained_quiz->id));
					
				foreach($questions as $cnt=>$question) {
					if(empty( $question->rank))  $question->rank = $cnt; // just in case this is empty for some reason
								
					// "text" becomes textarea in WatuPRO
					if($question->qtype == 'text') $question->qtype = 'textarea';		
					
					$wpdb->query($wpdb->prepare("INSERT INTO ".WATUPRO_QUESTIONS." SET
						exam_id=%d, question=%s, answer_type=%s, sort_order=%d",
						$quiz_id, stripslashes($question->question), $question->qtype, $question->rank));
						
					$qid = $wpdb->insert_id;
					$questions[$cnt]->watupro_question_id = $qid;
					
					foreach($choices as $ct=>$choice) {
						if($choice->question_id != $question->id) continue;
						$wpdb->query($wpdb->prepare("INSERT INTO ".WATUPRO_ANSWERS." SET
							question_id=%d, answer=%s, correct=%d, point=%f",
							$qid, stripslashes($choice->choice), $choice->is_correct, $choice->points));
						$cid = $wpdb->insert_id;	
						$choices[$ct]->watupro_choice_id = $cid;
					}	
				}	
				
				// transfer chained logic
				foreach($choices as $choice) {
					if(is_numeric($choice->goto)) {
						// figure out the new goto
						foreach($questions as $question) {
							if($choice->goto == $question->id) {
								$choice->goto = $question->watupro_question_id;
								break;
							}
						}
					}
					// otherwise keep as is
					
					$wpdb->query($wpdb->prepare("INSERT INTO ".WCHAINED_CHAINS." SET
							exam_id=%d, choice_id=%d, goto=%s",
							$quiz_id, $choice->watupro_choice_id, $choice->goto));
				}
				
				// copy grades
				$results = $wpdb->get_results($wpdb->prepare("SELECT * FROM {$wpdb->prefix}chained_results
					WHERE quiz_id=%d ORDER BY id", $chained_quiz->id));
				
				// array of IDs that match result IDs from chained quiz with the transferred grade IDs
				$grade_ids = array();	
					
				foreach($results as $result) {
					$wpdb->query($wpdb->prepare("INSERT INTO ".WATUPRO_GRADES." SET
						exam_id=%d, gfrom=%f, gto=%f, gtitle=%s, gdescription=%s",
						$quiz_id, $result->points_bottom, $result->points_top, 
						stripslashes($result->title), stripslashes($result->description)));
					$gid = $wpdb->insert_id;
					$grade_ids[$result->id] = $qid;	
				}	
				
				// transfer results (basic, no details)
				$takings = $wpdb->get_results($wpdb->prepare("SELECT tC.*, tR.title as result 
					FROM ".$wpdb->prefix."chained_completed tC 
					LEFT JOIN {$wpdb->prefix}chained_results tR ON tR.id = tC.result_id 
					WHERE tC.quiz_id=%d ORDER BY tC.id", $chained_quiz->id));
					
				foreach($takings as $taking) {
					$wpdb->query($wpdb->prepare("INSERT INTO ".WATUPRO_TAKEN_EXAMS." SET
						exam_id=%d, points=%f, result=%s, grade_id=%d, date=%s, start_time=%s, end_time=%s, 
						ip=%s, user_id=%d, details=%s, email=%s",
						$quiz_id, $taking->points, $taking->result, $taking->result_id, 
						date('Y-m-d', strtotime($taking->datetime)), $taking->datetime, $taking->datetime, $taking->ip,
						$taking->user_id, $taking->snapshot, $taking->email));
						
					// transfer detailed answers NYI and maybe never	
				}	
				
				// change shortcodes
				if(!empty($_POST['change_shortcodes'])) {
					$wpdb->query("UPDATE {$wpdb->posts} SET 
						post_content = REPLACE(post_content, '[chained-quiz ".$chained_quiz->id."]', '[watupro_chained quiz_id=".$quiz_id."]')"); 
				}				
			} // end foreach quiz
			
			$success = count($chained_quizzes)." quizzes transferred successfully.";
		}	
	} // end chained quiz integration
	
	// select logs	
	if($enable_log) {
		$logs = $wpdb->get_results("SELECT * FROM ".WCHAINED_LOGS." ORDER BY ID DESC LIMIT 1000");
	}
	
	$date_format = get_option('date_format').' '.get_option('time_format');
	$button_text = get_option('wchained_button_text');
	include(WCHAINED_PATH.'/views/options.html.php');
}

function wchained_menu() {
	$manage_caps = current_user_can('manage_options')?'manage_options':'watupro_manage_exams';
	add_submenu_page('watupro_exams', 'Chained Logic', 'Chained Logic', $manage_caps, 'wchained_options', 'wchained_options');
}

// gets a short resume from a question;
function wchained_summary($question) {
	if(!empty($question->title)) return $question->title;	
	
	$words = preg_split('/\s/', strip_tags(stripslashes($question->question)));
	$words = array_slice($words, 0, 10);
	$summary = "(ID: ".$question->ID.") ".implode(" ", $words).'...';	
	return $summary;
}